@@ -124,6 +124,7 @@ static bool IMG_InitJPG(void)
 
     return true;
 }
+
 #if 0
 void IMG_QuitJPG(void)
 {
@@ -343,89 +344,106 @@ static void output_no_message(j_common_ptr cinfo)
     (void)cinfo;
 }
 
+struct loadjpeg_vars {
+    const char *error;
+    SDL_Surface *surface;
+    struct jpeg_decompress_struct cinfo;
+    struct my_error_mgr jerr;
+};
+
 /* Load a JPEG type image from an SDL datasource */
+static bool LIBJPEG_LoadJPG_IO(SDL_IOStream *src, struct loadjpeg_vars *vars)
+{
+    JSAMPROW rowptr[1];
+
+    /* Create a decompression structure and load the JPEG header */
+    vars->cinfo.err = lib.jpeg_std_error(&vars->jerr.errmgr);
+    vars->jerr.errmgr.error_exit = my_error_exit;
+    vars->jerr.errmgr.output_message = output_no_message;
+    if (setjmp(vars->jerr.escape)) {
+        /* If we get here, libjpeg found an error */
+        lib.jpeg_destroy_decompress(&vars->cinfo);
+        vars->error = "JPEG loading error";
+        return false;
+    }
+
+    lib.jpeg_create_decompress(&vars->cinfo);
+    jpeg_SDL_IO_src(&vars->cinfo, src);
+    lib.jpeg_read_header(&vars->cinfo, TRUE);
+
+    if (vars->cinfo.num_components == 4) {
+        /* Set 32-bit Raw output */
+        vars->cinfo.out_color_space = JCS_CMYK;
+        vars->cinfo.quantize_colors = FALSE;
+        lib.jpeg_calc_output_dimensions(&vars->cinfo);
+
+        /* Allocate an output surface to hold the image */
+        vars->surface = SDL_CreateSurface(vars->cinfo.output_width, vars->cinfo.output_height, SDL_PIXELFORMAT_BGRA32);
+    } else {
+        /* Set 24-bit RGB output */
+        vars->cinfo.out_color_space = JCS_RGB;
+        vars->cinfo.quantize_colors = FALSE;
+#ifdef FAST_JPEG
+        vars->cinfo.scale_num   = 1;
+        vars->cinfo.scale_denom = 1;
+        vars->cinfo.dct_method = JDCT_FASTEST;
+        vars->cinfo.do_fancy_upsampling = FALSE;
+#endif
+        lib.jpeg_calc_output_dimensions(&vars->cinfo);
+
+        /* Allocate an output surface to hold the image */
+        vars->surface = SDL_CreateSurface(vars->cinfo.output_width, vars->cinfo.output_height, SDL_PIXELFORMAT_RGB24);
+    }
+
+    if (!vars->surface) {
+        lib.jpeg_destroy_decompress(&vars->cinfo);
+        return false;
+    }
+
+    /* Decompress the image */
+    lib.jpeg_start_decompress(&vars->cinfo);
+    while (vars->cinfo.output_scanline < vars->cinfo.output_height) {
+        rowptr[0] = (JSAMPROW)(Uint8 *)vars->surface->pixels +
+                            vars->cinfo.output_scanline * vars->surface->pitch;
+        lib.jpeg_read_scanlines(&vars->cinfo, rowptr, (JDIMENSION) 1);
+    }
+    lib.jpeg_finish_decompress(&vars->cinfo);
+    lib.jpeg_destroy_decompress(&vars->cinfo);
+
+    return true;
+}
+
 SDL_Surface *IMG_LoadJPG_IO(SDL_IOStream *src)
 {
     Sint64 start;
-    struct jpeg_decompress_struct cinfo;
-    JSAMPROW rowptr[1];
-    SDL_Surface *surface = NULL;
-    struct my_error_mgr jerr;
+    struct loadjpeg_vars vars;
 
-    if ( !src ) {
+    if (!src) {
         /* The error message has been set in SDL_IOFromFile */
         return NULL;
     }
-    start = SDL_TellIO(src);
 
     if (!IMG_InitJPG()) {
         return NULL;
     }
 
-    /* Create a decompression structure and load the JPEG header */
-    cinfo.err = lib.jpeg_std_error(&jerr.errmgr);
-    jerr.errmgr.error_exit = my_error_exit;
-    jerr.errmgr.output_message = output_no_message;
-#ifdef _MSC_VER
-#pragma warning(disable:4611)   /* warning C4611: interaction between '_setjmp' and C++ object destruction is non-portable */
-#endif
-    if (setjmp(jerr.escape)) {
-        /* If we get here, libjpeg found an error */
-        lib.jpeg_destroy_decompress(&cinfo);
-        if ( surface != NULL ) {
-            SDL_DestroySurface(surface);
-        }
-        SDL_SeekIO(src, start, SDL_IO_SEEK_SET);
-        SDL_SetError("JPEG loading error");
-        return NULL;
+    start = SDL_TellIO(src);
+    SDL_zero(vars);
+
+    if (LIBJPEG_LoadJPG_IO(src, &vars)) {
+        return vars.surface;
     }
 
-    lib.jpeg_create_decompress(&cinfo);
-    jpeg_SDL_IO_src(&cinfo, src);
-    lib.jpeg_read_header(&cinfo, TRUE);
-
-    if (cinfo.num_components == 4) {
-        /* Set 32-bit Raw output */
-        cinfo.out_color_space = JCS_CMYK;
-        cinfo.quantize_colors = FALSE;
-        lib.jpeg_calc_output_dimensions(&cinfo);
-
-        /* Allocate an output surface to hold the image */
-        surface = SDL_CreateSurface(cinfo.output_width, cinfo.output_height, SDL_PIXELFORMAT_BGRA32);
-    } else {
-        /* Set 24-bit RGB output */
-        cinfo.out_color_space = JCS_RGB;
-        cinfo.quantize_colors = FALSE;
-#ifdef FAST_JPEG
-        cinfo.scale_num   = 1;
-        cinfo.scale_denom = 1;
-        cinfo.dct_method = JDCT_FASTEST;
-        cinfo.do_fancy_upsampling = FALSE;
-#endif
-        lib.jpeg_calc_output_dimensions(&cinfo);
-
-        /* Allocate an output surface to hold the image */
-        surface = SDL_CreateSurface(cinfo.output_width, cinfo.output_height, SDL_PIXELFORMAT_RGB24);
+    /* this may clobber a set error if seek fails: don't care. */
+    SDL_SeekIO(src, start, SDL_IO_SEEK_SET);
+    if (vars.surface) {
+        SDL_DestroySurface(vars.surface);
+    }
+    if (vars.error) {
+        SDL_SetError("%s", vars.error);
     }
 
-    if ( surface == NULL ) {
-        lib.jpeg_destroy_decompress(&cinfo);
-        SDL_SeekIO(src, start, SDL_IO_SEEK_SET);
-        SDL_SetError("Out of memory");
-        return NULL;
-    }
-
-    /* Decompress the image */
-    lib.jpeg_start_decompress(&cinfo);
-    while ( cinfo.output_scanline < cinfo.output_height ) {
-        rowptr[0] = (JSAMPROW)(Uint8 *)surface->pixels +
-                            cinfo.output_scanline * surface->pitch;
-        lib.jpeg_read_scanlines(&cinfo, rowptr, (JDIMENSION) 1);
-    }
-    lib.jpeg_finish_decompress(&cinfo);
-    lib.jpeg_destroy_decompress(&cinfo);
-
-    return surface;
+    return NULL;
 }
 
 #define OUTPUT_BUFFER_SIZE   4096
